# JavaScript Example: Creating an Invoice

This example shows how to create an invoice using JavaScript with fetch API.

```javascript
/**
 * Sphere API - Create Invoice Example
 */

// API credentials
const merchantId = 'your_merchant_id';
const apiKey = 'your_api_key';

// API endpoint
const apiUrl = 'https://api.coinssend.com/v1/invoices';

// Invoice data
const invoiceData = {
  'order_id': 'order_' + Date.now(), // Use a unique order ID
  'amount': '100.50', // Amount in USD
  'is_customer_fee': true // Customer pays the fee
};

// Generate signature
const jsonData = JSON.stringify(invoiceData);
const base64Data = btoa(jsonData);
const signature = md5(base64Data + apiKey); // You'll need an MD5 library

// Make the API request
fetch(apiUrl, {
  method: 'POST',
  headers: {
    'Content-Type': 'application/json',
    'Merchant': merchantId,
    'Sign': signature
  },
  body: jsonData
})
.then(response => response.json())
.then(data => {
  if (data.status === 'success') {
    console.log("Invoice created successfully!");
    console.log("Payment URL: " + data.data.url);
    console.log("Invoice Code: " + data.data.code);
    
    // Redirect the user to the payment URL
    // window.location.href = data.data.url;
  } else {
    console.error("Error: " + data.error.message);
  }
})
.catch(error => {
  console.error("Request failed: " + error);
});

/**
 * Simple MD5 function for browser environments
 * Note: For production, use a proper crypto library
 */
function md5(string) {
  // In a real implementation, use a proper MD5 library
  // This is just a placeholder
  return CryptoJS.MD5(string).toString();
}
```

## Using with Axios

For a more modern approach using Axios:

```javascript
// Install axios: npm install axios
import axios from 'axios';
import md5 from 'md5'; // Install md5: npm install md5

// API credentials
const merchantId = 'your_merchant_id';
const apiKey = 'your_api_key';

// Invoice data
const invoiceData = {
  order_id: 'order_' + Date.now(),
  amount: '100.50',
  is_customer_fee: true
};

// Generate signature
const jsonData = JSON.stringify(invoiceData);
const base64Data = Buffer.from(jsonData).toString('base64');
const signature = md5(base64Data + apiKey);

// Create HTTP client
const client = axios.create({
  baseURL: 'https://api.coinssend.com/v1/',
  headers: {
    'Content-Type': 'application/json',
    'Merchant': merchantId,
    'Sign': signature
  }
});

// Make request
client.post('invoices', invoiceData)
  .then(response => {
    const responseData = response.data;
    
    if (responseData.status === 'success') {
      console.log("Invoice created successfully!");
      console.log("Payment URL: " + responseData.data.url);
      console.log("Invoice Code: " + responseData.data.code);
    } else {
      console.error("Error: " + responseData.error.message);
    }
  })
  .catch(error => {
    if (error.response) {
      const errorResponse = error.response.data;
      console.error("API Error: " + (errorResponse.error?.message || 'Unknown error'));
    } else {
      console.error("Connection Error: " + error.message);
    }
  });
```

## Reusable API Client Class

For multiple API calls, create a reusable client class:

```javascript
class SphereApiClient {
  constructor(merchantId, apiKey, baseUrl = 'https://api.coinssend.com/v1/') {
    this.merchantId = merchantId;
    this.apiKey = apiKey;
    this.baseUrl = baseUrl;
  }
  
  async createInvoice(orderId, amount, isCustomerFee = false) {
    const data = {
      order_id: orderId,
      amount: amount,
      is_customer_fee: isCustomerFee
    };
    
    return this.sendRequest('POST', 'invoices', data);
  }
  
  async getInvoice(invoiceCode) {
    return this.sendRequest('GET', `invoices/${invoiceCode}`);
  }
  
  async initiateWithdrawal(network, coin, amount, toAddress) {
    const data = {
      network: network,
      coin: coin,
      amount: amount,
      to_address: toAddress
    };
    
    return this.sendRequest('POST', 'withdrawals', data);
  }
  
  async sendRequest(method, endpoint, data = null) {
    const url = this.baseUrl + endpoint;
    const jsonData = data ? JSON.stringify(data) : '';
    const base64Data = data ? btoa(jsonData) : '';
    const signature = data ? md5(base64Data + this.apiKey) : '';
    
    const headers = {
      'Content-Type': 'application/json',
      'Merchant': this.merchantId
    };
    
    if (signature) {
      headers['Sign'] = signature;
    }
    
    const options = {
      method: method,
      headers: headers
    };
    
    if (jsonData) {
      options.body = jsonData;
    }
    
    try {
      const response = await fetch(url, options);
      const responseData = await response.json();
      
      if (response.ok) {
        return responseData;
      } else {
        throw new Error(responseData.error?.message || 'API Error');
      }
    } catch (error) {
      throw error;
    }
  }
}

// Usage
try {
  const client = new SphereApiClient('your_merchant_id', 'your_api_key');
  const result = await client.createInvoice('order_' + Date.now(), '100.50', true);
  
  console.log("Invoice created successfully!");
  console.log("Payment URL: " + result.data.url);
  console.log("Invoice Code: " + result.data.code);
} catch (error) {
  console.error("Error: " + error.message);
}
```